require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_environment',
  dscmeta_resource_friendly_name: 'Environment',
  dscmeta_resource_name: 'MSFT_EnvironmentResource',
  dscmeta_resource_implementation: 'Class',
  dscmeta_module_name: 'PSDscResources',
  dscmeta_module_version: '2.12.0.0',
  docs: 'The DSC Environment resource type.
         Automatically generated from version 2.12.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',

      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'present', 'Absent', 'absent']]",
      desc: 'Specifies if the environment varaible should exist.',


      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_value: {
      type: 'Optional[String]',
      desc: 'The desired value for the environment variable.',


      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'The name of the environment variable for which you want to ensure a specific state.',

      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_target: {
      type: "Optional[Array[Enum['Process', 'process', 'Machine', 'machine']]]",
      desc: 'Indicates the target where the environment variable should be set.',


      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_path: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether or not the environment variable is the Path variable.',


      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
