require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_script',
  dscmeta_resource_friendly_name: 'Script',
  dscmeta_resource_name: 'MSFT_ScriptResource',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'PSDscResources',
  dscmeta_module_version: '2.12.0.0',
  docs: 'The DSC Script resource type.
         Automatically generated from version 2.12.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',

      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_getscript: {
      type: 'String',
      desc: 'A string that can be used to create a PowerShell script block that retrieves the current state of the resource.',

      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_setscript: {
      type: 'String',
      desc: 'A string that can be used to create a PowerShell script block that sets the resource to the desired state.',

      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_testscript: {
      type: 'String',
      desc: 'A string that can be used to create a PowerShell script block that validates whether or not the resource is in the desired state.',

      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'The credential of the user account to run the script under if needed.',

      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_result: {
      type: 'Optional[String]',
      desc: 'The result from the GetScript script block.',

      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    ensurable: {
      type: 'Boolean[false]',
      desc: 'Default attribute added to all dsc types without an ensure property. This resource is not ensurable.',
      default: false,

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
